﻿namespace UFIDA.U9.Cust.Kusi.CJJ.RestApi
{
	using System;
	using System.Collections.Generic;
	using System.Text;
    using UFSoft.UBF.AopFrame;
    using UFSoft.UBF.Util.Context;

    /// <summary>
    /// GetBarCodeUsedWhBinInfoSV partial 
    /// </summary>	
	public partial class GetBarCodeUsedWhBinInfoSV 
	{	
		internal BaseStrategy Select()
		{
			return new GetBarCodeUsedWhBinInfoSVImpementStrategy();	
		}		
	}
	
	#region  implement strategy	
	/// <summary>
	/// Impement Implement
	/// 
	/// </summary>	
	internal partial class GetBarCodeUsedWhBinInfoSVImpementStrategy : BaseStrategy
	{
		public GetBarCodeUsedWhBinInfoSVImpementStrategy() { }

		public override object Do(object obj)
		{						
			GetBarCodeUsedWhBinInfoSV bpObj = (GetBarCodeUsedWhBinInfoSV)obj;

			List<BarCodeUsedWhBinDTOData> result = new List<BarCodeUsedWhBinDTOData>();

			foreach (long id in bpObj.BarCodeUsedIDs)
			{
				BC.BaseData.BarCodeUsed barCodeUsed = BC.BaseData.BarCodeUsed.Finder.FindByID(id);
				if (barCodeUsed != null)
				{
					result.Add(GetWhBin(barCodeUsed));

                }
			}

			return result;
		}		

		private BarCodeUsedWhBinDTOData GetWhBin(BC.BaseData.BarCodeUsed barCodeUsed)
		{
			BarCodeUsedWhBinDTOData result = new BarCodeUsedWhBinDTOData();

			result.BarCodeUsedID = barCodeUsed.ID;
            if (barCodeUsed.UseDocLine != null)
            {
                switch (barCodeUsed.UseDocLine.EntityType)
                {
                    case "UFIDA.U9.Complete.RCVRpt.RcvRptDocLine":
                        FromRcvRptDocLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.PM.Rcv.RcvLine":
                        FromRcvLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.InvDoc.MiscRcv.MiscRcvTransL":
                        FromMiscRcvTransL(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.InvDoc.TransferIn.TransInLine":
                        FromTransInLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.InvDoc.InventorySheet.InventSheetLine":
                        FromInventSheetLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.SM.Ship.ShipLine":
                        FromShipLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.InvDoc.MiscShip.MiscShipmentL":
                        FromMiscShipmentL(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA::U9::MO::Issue::IssueDocLine":
                        FromIssueDocLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA::U9::MO::Issue::IssueDocLineSum":
                        FromIssueDocLineSum(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                    case "UFIDA.U9.InvDoc.WhInit.WhInitLine":
                        FromWhInitLine(barCodeUsed.UseDocLine.EntityID, result);
                        break;
                }
            }

            return result;
        }

        private string GetBinBarCode(long binID)
        {
            BC.BaseData.BarCode barCode = BC.BaseData.BarCode.Finder.Find("CreateEntity.EntityType=@Type and CreateEntity.EntityID=@EntityID",
                new UFSoft.UBF.PL.OqlParam[]
                {
                    new UFSoft.UBF.PL.OqlParam("UFIDA.U9.CBO.SCM.Bin.Bin"),
                    new UFSoft.UBF.PL.OqlParam(binID)
                });
            if (barCode != null)
            {
                return barCode.Code;
            }

            return "";
        }

        private void FromRcvRptDocLine(long id, BarCodeUsedWhBinDTOData info)
        {
            Complete.RCVRpt.RcvRptDocLine line = Complete.RCVRpt.RcvRptDocLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.Bin != null)
                {
                    info.BinID = line.Bin.ID;
                    info.BinCode = line.Bin.Code;
                    info.BinName = line.Bin.Name;
                    if (line.Bin.Org != null)
                    {
                        info.BinOrgCode = line.Bin.Org.Code;
                    }
                    info.BinBarCode = GetBinBarCode(line.Bin.ID);
                }
            }
        }

        private void FromRcvLine(long id, BarCodeUsedWhBinDTOData info)
        {
            PM.Rcv.RcvLine line = PM.Rcv.RcvLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.RcvLineLocations.Count > 0)
                {
                    if (line.RcvLineLocations[0].Location != null)
                    {
                        info.BinID = line.RcvLineLocations[0].Location.ID;
                        info.BinCode = line.RcvLineLocations[0].Location.Code;
                        info.BinName = line.RcvLineLocations[0].Location.Name;

                        if (line.RcvLineLocations[0].Location.Org != null)
                        {
                            info.BinOrgCode = line.RcvLineLocations[0].Location.Org.Code;
                        }
                        info.BinBarCode = GetBinBarCode(line.RcvLineLocations[0].Location.ID);
                    }
                }
            }
        }

        private void FromMiscRcvTransL(long id, BarCodeUsedWhBinDTOData info)
        {
            InvDoc.MiscRcv.MiscRcvTransL line = InvDoc.MiscRcv.MiscRcvTransL.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.MiscRcvTransBins.Count > 0)
                {
                    if (line.MiscRcvTransBins[0].BinInfo != null && line.MiscRcvTransBins[0].BinInfo.Bin != null)
                    {
                        info.BinID = line.MiscRcvTransBins[0].BinInfo.Bin.ID;
                        info.BinCode = line.MiscRcvTransBins[0].BinInfo.Bin.Code;
                        info.BinName = line.MiscRcvTransBins[0].BinInfo.Bin.Name;

                        if (line.MiscRcvTransBins[0].BinInfo.Bin.Org != null)
                        {
                            info.BinOrgCode = line.MiscRcvTransBins[0].BinInfo.Bin.Org.Code;
                        }
                        info.BinBarCode = GetBinBarCode(line.MiscRcvTransBins[0].BinInfo.Bin.ID);
                    }
                }
            }
        }

        private void FromTransInLine(long id, BarCodeUsedWhBinDTOData info)
        {
            InvDoc.TransferIn.TransInLine line = InvDoc.TransferIn.TransInLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.TransInWh != null)
                {
                    info.WhID = line.TransInWh.ID;
                    info.WhCode = line.TransInWh.Code;
                    info.WhName = line.TransInWh.Name;
                }

                if (line.TransInBins.Count > 0)
                {
                    if (line.TransInBins[0].BinInfo != null && line.TransInBins[0].BinInfo.Bin != null)
                    {
                        info.BinID = line.TransInBins[0].BinInfo.Bin.ID;
                        info.BinCode = line.TransInBins[0].BinInfo.Bin.Code;
                        info.BinName = line.TransInBins[0].BinInfo.Bin.Name;

                        if (line.TransInBins[0].BinInfo.Bin.Org != null)
                        {
                            info.BinOrgCode = line.TransInBins[0].BinInfo.Bin.Org.Code;
                        }
                        info.BinBarCode = GetBinBarCode(line.TransInBins[0].BinInfo.Bin.ID);
                    }
                }
            }
        }

        private void FromInventSheetLine(long id, BarCodeUsedWhBinDTOData info)
        {
            InvDoc.InventorySheet.InventSheetLine line = InvDoc.InventorySheet.InventSheetLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.InventorySheet.Wh != null)
                {
                    info.WhID = line.InventorySheet.Wh.ID;
                    info.WhCode = line.InventorySheet.Wh.Code;
                    info.WhName = line.InventorySheet.Wh.Name;
                }

                if (line.Bin != null && line.Bin.Bin != null)
                {
                    info.BinID = line.Bin.Bin.ID;
                    info.BinCode = line.Bin.Bin.Code;
                    info.BinName = line.Bin.Bin.Name;

                    if (line.Bin.Bin.Org != null)
                    {
                        info.BinOrgCode = line.Bin.Bin.Org.Code;
                    }
                    info.BinBarCode = GetBinBarCode(line.Bin.Bin.ID);
                }
            }
        }

        private void FromShipLine(long id, BarCodeUsedWhBinDTOData info)
        {
            SM.Ship.ShipLine line = SM.Ship.ShipLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.WH != null)
                {
                    info.WhID = line.WH.ID;
                    info.WhCode = line.WH.Code;
                    info.WhName = line.WH.Name;
                }

                if (line.ShipLineLocations.Count > 0)
                {
                    if (line.ShipLineLocations[0].Location != null)
                    {
                        info.BinID = line.ShipLineLocations[0].Location.ID;
                        info.BinCode = line.ShipLineLocations[0].Location.Code;
                        info.BinName = line.ShipLineLocations[0].Location.Name;

                        if (line.ShipLineLocations[0].Location.Org != null)
                        {
                            info.BinOrgCode = line.ShipLineLocations[0].Location.Org.Code;
                        }
                        info.BinBarCode = GetBinBarCode(line.ShipLineLocations[0].Location.ID);
                    }
                }
            }
        }

        private void FromMiscShipmentL(long id, BarCodeUsedWhBinDTOData info)
        {
            InvDoc.MiscShip.MiscShipmentL line = InvDoc.MiscShip.MiscShipmentL.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.MiscShipBins.Count > 0)
                {
                    if (line.MiscShipBins[0].BinInfo != null && line.MiscShipBins[0].BinInfo.Bin != null)
                    {
                        info.BinID = line.MiscShipBins[0].BinInfo.Bin.ID;
                        info.BinCode = line.MiscShipBins[0].BinInfo.Bin.Code;
                        info.BinName = line.MiscShipBins[0].BinInfo.Bin.Name;

                        if (line.MiscShipBins[0].BinInfo.Bin.Org != null)
                        {
                            info.BinOrgCode = line.MiscShipBins[0].BinInfo.Bin.Org.Code;
                        }
                        info.BinBarCode = GetBinBarCode(line.MiscShipBins[0].BinInfo.Bin.ID);
                    }
                }
            }
        }

        private void FromIssueDocLine(long id, BarCodeUsedWhBinDTOData info)
        {
            MO.Issue.IssueDocLine line = MO.Issue.IssueDocLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.Bin != null)
                {
                    info.BinID = line.Bin.ID;
                    info.BinCode = line.Bin.Code;
                    info.BinName = line.Bin.Name;

                    if (line.Bin.Org != null)
                    {
                        info.BinOrgCode = line.Bin.Org.Code;
                    }
                    info.BinBarCode = GetBinBarCode(line.Bin.ID);
                }
            }
        }

        private void FromIssueDocLineSum(long id, BarCodeUsedWhBinDTOData info)
        {
            MO.Issue.IssueDocLineSum line = MO.Issue.IssueDocLineSum.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.Bin != null)
                {
                    info.BinID = line.Bin.ID;
                    info.BinCode = line.Bin.Code;
                    info.BinName = line.Bin.Name;

                    if (line.Bin.Org != null)
                    {
                        info.BinOrgCode = line.Bin.Org.Code;
                    }
                    info.BinBarCode = GetBinBarCode(line.Bin.ID);
                }
            }
        }

        private void FromWhInitLine(long id, BarCodeUsedWhBinDTOData info)
        {
            InvDoc.WhInit.WhInitLine line = InvDoc.WhInit.WhInitLine.Finder.FindByID(id);
            if (line != null)
            {
                if (line.Wh != null)
                {
                    info.WhID = line.Wh.ID;
                    info.WhCode = line.Wh.Code;
                    info.WhName = line.Wh.Name;
                }

                if (line.WhInitLineBins.Count > 0)
                {
                    if (line.WhInitLineBins[0].Bin != null && line.WhInitLineBins[0].Bin.Bin != null)
                    {
                        info.BinID = line.WhInitLineBins[0].Bin.Bin.ID;
                        info.BinCode = line.WhInitLineBins[0].Bin.Bin.Code;
                        info.BinName = line.WhInitLineBins[0].Bin.Bin.Name;

                        if (line.WhInitLineBins[0].Bin.Bin.Org != null)
                        {
                            info.BinOrgCode = line.WhInitLineBins[0].Bin.Bin.Org.Code;
                        }
                        info.BinBarCode = GetBinBarCode(line.WhInitLineBins[0].Bin.Bin.ID);
                    }
                }
            }
        }
    }

	#endregion
	
	
}