﻿namespace UFIDA.U9.Cust.Kusi.CJJ.RestApi
{
	using System;
	using System.Collections.Generic;
	using System.Data;
	using System.Text;
	using UFIDA.U9.BC.Common;
	using UFSoft.UBF.AopFrame;	
	using UFSoft.UBF.Util.Context;
	using UFSoft.UBF.Util.DataAccess;

	/// <summary>
	/// GetItemBarCodeDetails partial 
	/// </summary>	
	public partial class GetItemBarCodeDetails 
	{	
		internal BaseStrategy Select()
		{
			return new GetItemBarCodeDetailsImpementStrategy();	
		}		
	}
	
	#region  implement strategy	
	/// <summary>
	/// Impement Implement
	/// 
	/// </summary>	
	internal partial class GetItemBarCodeDetailsImpementStrategy : BaseStrategy
	{
		public GetItemBarCodeDetailsImpementStrategy() { }

		public override object Do(object obj)
		{						
			GetItemBarCodeDetails bpObj = (GetItemBarCodeDetails)obj;

			List<BarCodeDetailInfoData> result = new List<BarCodeDetailInfoData>();

			foreach(var barCodeInfo in bpObj.BarCodeInfo)
			{
				BarCodeDetailInfoData data = GetDetialInfo(barCodeInfo.BarCode);
				if (data != null)
				{
					result.Add(data);
				}
			}

			return result;
		}
		
		private BarCodeDetailInfoData GetDetialInfo(string code)
		{
			BC.BaseData.BarCode barCode = BC.BaseData.BarCode.Finder.Find("Code=@Code", new UFSoft.UBF.PL.OqlParam[]{ new UFSoft.UBF.PL.OqlParam(code) });
			if (barCode != null)
			{
				if (barCode.BarCodeType != BarCodeTypeEnum.ItemBarCode)
				{
					throw new Exception("该条码不是料品条码，请扫料品条码");
				}

				BarCodeDetailInfoData data = new BarCodeDetailInfoData();
				data.BarCode = barCode.Code;
				data.ItemCode = barCode.Segment1;
				data.ItemName = GetItemName(data.ItemCode);
				data.Seiban = barCode.Segment2;
				data.Project = GetProject(barCode);
				data.UseDateils = GetUsedDetails(barCode.ID, code);
				data.PackageInfos = GetPackageInfos(code);

				return data;
			}

			return null;
        }

		private string GetItemName(string itemCode)
		{
			CBO.SCM.Item.ItemMaster itemMaster = CBO.SCM.Item.ItemMaster.Finder.Find("Code=@Code", new UFSoft.UBF.PL.OqlParam[] { new UFSoft.UBF.PL.OqlParam(itemCode) });
			if (itemMaster != null)
			{
				return itemMaster.Name;
			}

            return "";
		}

		private string GetProject(BC.BaseData.BarCode barCode)
		{
			foreach(var line in barCode.BarCodeLine)
			{
				if (line.DimensionType == UFIDA.U9.BC.Common.DimensionTypeEnum.Project)
				{
					return (string)barCode.GetValue(string.Format("Segment{0}", line.SegmentSeq));
				}
			}

			return "";
		}

		private List<PackageInfoData> GetPackageInfos(string code)
		{
			List<PackageInfoData> packageInfos = new List<PackageInfoData>();

            DataParamList dataParams = new DataParamList
            {
                DataParamFactory.CreateInput("BarCodes", code, DbType.String)
            };

            DataAccessor.RunSP("SP_Cust_PDA_GetBarCodeDetail_PackageBarCodes", dataParams, out DataSet dataSet);
            if (dataSet != null && dataSet.Tables.Count > 0 && dataSet.Tables[0].Rows.Count > 0)
            {
				foreach(DataRow row in dataSet.Tables[0].Rows)
				{
					PackageInfoData packageInfo = new PackageInfoData();
					packageInfo.PackageSize = Convert.ToString(row["PackageCount"]);
                    packageInfo.Sequence = Convert.ToString(row["Sequence"]);
                    packageInfo.PartName = Convert.ToString(row["PartName"]);
					packageInfo.BarCode = Convert.ToString(row["Code"]);

                    packageInfos.Add(packageInfo);
                }
            }

			return packageInfos;
        }

		private List<UsedDetailData> GetUsedDetails(long barCodeMasterID, string code)
		{
			List<UsedDetailData> result = new List<UsedDetailData>();

			string oql = GetUsedDetailOql(code);
            UFIDA.U9.BC.BaseData.BarCodeUsed.EntityList usedInfos = UFIDA.U9.BC.BaseData.BarCodeUsed.Finder.FindAll(oql, new UFSoft.UBF.PL.OqlParam[] { new UFSoft.UBF.PL.OqlParam(barCodeMasterID) });
			foreach(var info in usedInfos)
			{
				UsedDetailData data = new UsedDetailData();
				data.UseDate = info.UseDate;
				data.CreateBy = info.CreatedBy;
				data.Direct = info.Direct.Value;
				data.BinInfo = new List<BinDTOData>();
				SetDocInfo(data, info);

				result.Add(data);
			}

            return result;
		}

		private string GetUsedDetailOql(string code)
		{
			StringBuilder sb = new StringBuilder();

			if (code.StartsWith("C") && code.Length == 16)
			{
				sb.Append("CopyOfBarCodeApp not in (7, 9) and BarCodeRule.Code='LPITEM11' ");
            }
			else
			{
                sb.Append("CopyOfBarCodeApp not in (0, 7, 9) ");
            }

            sb.Append("and UseDocLine.EntityType in (");
            sb.Append("'UFIDA.U9.Complete.RCVRpt.RcvRptDocLine', ");
            sb.Append("'UFIDA.U9.PM.Rcv.RcvLine', ");
            sb.Append("'UFIDA.U9.InvDoc.MiscRcv.MiscRcvTransL', ");
            sb.Append("'UFIDA.U9.InvDoc.TransferIn.TransInLine', ");
            sb.Append("'UFIDA.U9.InvDoc.InventorySheet.InventSheetLine', ");
            sb.Append("'UFIDA.U9.SM.Ship.ShipLine', "); 
            sb.Append("'UFIDA.U9.InvDoc.MiscShip.MiscShipmentL', ");
            sb.Append("'UFIDA::U9::MO::Issue::IssueDocLine', ");
            sb.Append("'UFIDA::U9::MO::Issue::IssueDocLineSum', ");
            sb.Append("'UFIDA.U9.InvDoc.WhInit.WhInitLine'");
            sb.Append(") and BarCodeMaster.ID=@BarCodeMasterID order by UseDate desc");


            //else
            //{
            //	sb.Append("CopyOfBarCodeApp not in (0, 7, 9) ");
            //             sb.Append("and UseDoc.EntityType in (");
            //             sb.Append("'UFIDA.U9.Complete.RCVRpt.RcvRptDoc', ");
            //             sb.Append("'UFIDA.U9.PM.Rcv.Receivement', ");
            //             sb.Append("'UFIDA.U9.InvDoc.MiscRcv.MiscRcvTrans', ");
            //             sb.Append("'UFIDA.U9.InvDoc.TransferIn.TransferIn', ");
            //             sb.Append("'UFIDA.U9.InvDoc.InventorySheet.InventorySheet', ");
            //             sb.Append("'UFIDA.U9.SM.Ship.Ship', ");
            //             sb.Append("'UFIDA.U9.InvDoc.MiscShip.MiscShipment', ");
            //             sb.Append("'UFIDA.U9.MO.Issue.IssueDoc'");
            //             sb.Append(")");
            //         }



            return sb.ToString();
		}

		private void SetDocInfo(UsedDetailData data, BC.BaseData.BarCodeUsed info)
		{
			data.DocNo = info.DocNo;
			data.WhCode = "";
			data.WhName = "";
			data.SrcDocNo = "";
			if (info.UseDocLine != null)
			{
				switch (info.UseDocLine.EntityType)
				{
					case "UFIDA.U9.Complete.RCVRpt.RcvRptDocLine":
						FromRcvRptDocLine(data, info);
                        break;
                    case "UFIDA.U9.PM.Rcv.RcvLine":
                        FromRcvLine(data, info);
                        break;
					case "UFIDA.U9.InvDoc.MiscRcv.MiscRcvTransL":
                        FromMiscRcvTransL(data, info);
                        break;
					case "UFIDA.U9.InvDoc.TransferIn.TransInLine":
						FromTransInLine(data, info);
                        break;
					case "UFIDA.U9.InvDoc.InventorySheet.InventSheetLine":
						FromInventSheetLine(data, info);
                        break;
					case "UFIDA.U9.SM.Ship.ShipLine":
						FromShipLine(data, info);
                        break;
					case "UFIDA.U9.InvDoc.MiscShip.MiscShipmentL":
						FromMiscShipmentL(data, info);
                        break;
					case "UFIDA::U9::MO::Issue::IssueDocLine":
						FromIssueDocLine(data, info);
                        break;
                    case "UFIDA::U9::MO::Issue::IssueDocLineSum":
                        FromIssueDocLineSum(data, info);
                        break;
                    case "UFIDA.U9.InvDoc.WhInit.WhInitLine":
                        FromWhInitLine(data, info);
                        break;
                }
			}
		}

		private void FromRcvRptDocLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
		{
			data.SrcDocType = "入库单";
			Complete.RCVRpt.RcvRptDocLine line = Complete.RCVRpt.RcvRptDocLine.Finder.FindByID(info.UseDocLine.EntityID);
			if (line != null)
			{
				if (line.MO != null)
				{
					data.SrcDocNo = line.MO.DocNo;
				}

				if (line.Wh != null)
				{
					data.WhCode = line.Wh.Code;
					data.WhName = line.Wh.Name;
				}

				if (line.Bin != null)
				{
					data.BinInfo.Add(new BinDTOData() 
					{ 
						Code = line.Bin.Code,
						Name = line.Bin.Name
					});
				}
			}
        }

		private void FromRcvLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
		{
			data.SrcDocType = "收货单";
			PM.Rcv.RcvLine line = PM.Rcv.RcvLine.Finder.FindByID(info.UseDocLine.EntityID);
			if (line != null)
			{
				if (line.SrcDoc != null)
				{
					data.SrcDocNo = line.SrcDoc.SrcDocNo;
				}

				if (line.Wh != null)
				{
					data.WhCode = line.Wh.Code;
					data.WhName = line.Wh.Name;
				}

				foreach (var bin in line.RcvLineLocations)
				{
					if (bin.Location != null)
					{
						data.BinInfo.Add(new BinDTOData 
						{ 
							Code = bin.Location.Code,
							Name = bin.Location.Name
						});
					}
				}
			}
        }

        private void FromMiscRcvTransL(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "库存杂收单";
            InvDoc.MiscRcv.MiscRcvTransL line = InvDoc.MiscRcv.MiscRcvTransL.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
				if (line.SrcDoc != null)
				{
					data.SrcDocNo = line.SrcDoc.SrcDocNo;
				}

				if (line.Wh != null)
				{
					data.WhCode = line.Wh.Code;
					data.WhName = line.Wh.Name;
				}

				foreach (var bin in line.MiscRcvTransBins)
				{
					if (bin.BinInfo != null && bin.BinInfo.Bin != null)
					{
						data.BinInfo.Add(new BinDTOData 
						{ 
							Code = bin.BinInfo.Bin.Code,
							Name = bin.BinInfo.Bin.Name
						});
					}
				}
            }
        }

        private void FromTransInLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "库存调入单";
            UFIDA.U9.InvDoc.TransferIn.TransInLine line = UFIDA.U9.InvDoc.TransferIn.TransInLine.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
				if (line.SrcDocInfo != null)
				{
					data.SrcDocNo = line.SrcDocInfo.SrcDocNo;
				}

				if (line.TransInWh != null)
				{
					data.WhCode = line.TransInWh.Code;
					data.WhName = line.TransInWh.Name;
				}

				foreach (var bin in line.TransInBins)
				{
					if (bin.BinInfo != null && bin.BinInfo.Bin != null)
					{
						data.BinInfo.Add(new BinDTOData 
						{ 
							Code = bin.BinInfo.Bin.Code,
							Name = bin.BinInfo.Bin.Name
						});
					}
				}
            }
        }

        private void FromInventSheetLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "日常盘点单";
            UFIDA.U9.InvDoc.InventorySheet.InventSheetLine line = UFIDA.U9.InvDoc.InventorySheet.InventSheetLine.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
				if (line.SrcDocInfo != null)
				{
					data.SrcDocNo = line.SrcDocInfo.SrcDocNo;
				}

				if (line.InventorySheet.Wh != null)
				{
					data.WhCode = line.InventorySheet.Wh.Code;
					data.WhName = line.InventorySheet.Wh.Name;
				}

				if (line.Bin != null && line.Bin.Bin != null)
				{
					data.BinInfo.Add(new BinDTOData 
					{ 
						Code = line.Bin.Bin.Code,
						Name = line.Bin.Bin.Name
					});
				}
            }
        }

        private void FromShipLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "出货单";
            UFIDA.U9.SM.Ship.ShipLine line = UFIDA.U9.SM.Ship.ShipLine.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
                data.SrcDocNo = line.SrcDocNo;

				if (line.WH != null)
				{
					data.WhCode = line.WH.Code;
					data.WhName = line.WH.Name;
				}

				foreach (var bin in line.ShipLineLocations)
				{
					if (bin.Location != null)
					{
						data.BinInfo.Add(new BinDTOData
						{
							Code = bin.Location.Code,
							Name = bin.Location.Name
						});
					}
				}
            }
        }

        private void FromMiscShipmentL(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "库存杂发单";
            UFIDA.U9.InvDoc.MiscShip.MiscShipmentL line = UFIDA.U9.InvDoc.MiscShip.MiscShipmentL.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
                if (line.SrcDoc != null)
                {
                    data.SrcDocNo = line.SrcDoc.SrcDocNo;
                }

                if (line.Wh != null)
                {
                    data.WhCode = line.Wh.Code;
                    data.WhName = line.Wh.Name;
                }

                foreach (var bin in line.MiscShipBins)
                {
                    if (bin.BinInfo != null && bin.BinInfo.Bin != null)
                    {
                        data.BinInfo.Add(new BinDTOData
                        {
                            Code = bin.BinInfo.Bin.Code,
                            Name = bin.BinInfo.Bin.Name
                        });
                    }
                }
            }          
        }

        private void FromIssueDocLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "领料单";
            UFIDA.U9.MO.Issue.IssueDocLine line = UFIDA.U9.MO.Issue.IssueDocLine.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
                if (line.SourceDoc != null)
                {
                    data.SrcDocNo = line.SourceDoc.SrcDocNo;
                }

                if (line.Wh != null)
                {
                    data.WhCode = line.Wh.Code;
                    data.WhName = line.Wh.Name;
                }

                if (line.Bin != null)
                {
                    data.BinInfo.Add(new BinDTOData
                    {
                        Code = line.Bin.Code,
                        Name = line.Bin.Name
                    });
                }
            }
        }

        private void FromIssueDocLineSum(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "领料单";
            UFIDA.U9.MO.Issue.IssueDocLineSum line = UFIDA.U9.MO.Issue.IssueDocLineSum.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
                if (line.IssueDoc.SourceDoc != null)
                {
                    data.SrcDocNo = line.IssueDoc.SourceDoc.SrcDocNo;
                }

                if (line.Wh != null)
                {
                    data.WhCode = line.Wh.Code;
                    data.WhName = line.Wh.Name;
                }

                if (line.Bin != null)
                {
                    data.BinInfo.Add(new BinDTOData
                    {
                        Code = line.Bin.Code,
                        Name = line.Bin.Name
                    });
                }
            }
        }

        private void FromWhInitLine(UsedDetailData data, BC.BaseData.BarCodeUsed info)
        {
            data.SrcDocType = "库存期初单";
            UFIDA.U9.InvDoc.WhInit.WhInitLine line = UFIDA.U9.InvDoc.WhInit.WhInitLine.Finder.FindByID(info.UseDocLine.EntityID);
            if (line != null)
            {
                data.SrcDocNo = "";

                if (line.Wh != null)
                {
                    data.WhCode = line.Wh.Code;
                    data.WhName = line.Wh.Name;
                }

                foreach (var binInfo in line.WhInitLineBins)
				{
					if (binInfo.Bin != null && binInfo.Bin.Bin != null)
					{
						data.BinInfo.Add(new BinDTOData
						{
							Code = binInfo.Bin.Bin.Code,
							Name = binInfo.Bin.Bin.Name
						});
					}
                }
            }
        }
    }

	#endregion
	
	
}