﻿namespace UFIDA.U9.Cust.Kusi.CJJ.RestApi
{
	using System;
	using System.Collections.Generic;
    using System.Reflection;
    using System.Text;
    using UFIDA.U9.Approval.Operations;
    using UFIDA.U9.Approval.Operations.Proxy;
    using UFIDA.U9.Base;
    using UFIDA.U9.Base.Doc;
    using UFIDA.U9.Base.PropertyTypes;
    using UFIDA.U9.BC.CalcData;
    using UFIDA.U9.BC.CalcData.Proxy;
    using UFIDA.U9.BC.Common;
    using UFIDA.U9.CBO.Pub.Controller;
    using UFIDA.U9.ISV.Mobile.Common;
    using UFIDA.U9.ISV.Mobile.SM;
    using UFIDA.U9.ISV.MobileApprovalSV;
    using UFIDA.U9.ISV.SM;
    using UFIDA.U9.ISV.SM.Proxy;
    using UFIDA.U9.SM;
    using UFIDA.U9.SM.Ship;
    using UFSoft.UBF.AopFrame;
    using UFSoft.UBF.Business;
    using UFSoft.UBF.PL;
    using UFSoft.UBF.PL.Engine.Cache;
    using UFSoft.UBF.Util.Context;

    /// <summary>
    /// ShipPickConfirmUpdateBarCodeFirstSV partial 
    /// </summary>	
	public partial class ShipPickConfirmUpdateBarCodeFirstSV 
	{	
		internal BaseStrategy Select()
		{
			return new ShipPickConfirmUpdateBarCodeFirstSVImpementStrategy();	
		}		
	}
	
	#region  implement strategy	
	/// <summary>
	/// Impement Implement
	/// 
	/// </summary>	
	internal partial class ShipPickConfirmUpdateBarCodeFirstSVImpementStrategy : BaseStrategy
	{
		public ShipPickConfirmUpdateBarCodeFirstSVImpementStrategy() { }

		public override object Do(object obj)
		{						
			ShipPickConfirmUpdateBarCodeFirstSV bpObj = (ShipPickConfirmUpdateBarCodeFirstSV)obj;
			foreach (ShipPickConfirmBCDTOData dto in bpObj.DocInfo)
            {
                this.UpdateBarCodeState(dto);
                this.CreateItemBarCodeUseDetail(dto);
                this.ConfirmShip(dto);
            }
            return null;
        }

        /// <summary>
        /// 确认出货单
        /// </summary>
        /// <param name="dto"></param>
        private void ConfirmShip(ShipPickConfirmBCDTOData dto)
        {
            this.CheckShip(dto);
            this.SetDefaultValue(dto);
            this.ModifyShip(dto);
            this.ApproveShip(dto);
        }

        /// <summary>
        /// 数据检查
        /// </summary>
        /// <param name="dto"></param>
        private void CheckShip(ShipPickConfirmBCDTOData dto)
        {
            foreach (ShipPickConfirmBCLineDTOData shipPickConfirmBCLineDTOData in dto.DocLines)
            {
                ShipLine shipLine = ShipLine.Finder.FindByID(shipPickConfirmBCLineDTOData.DocLineKey);
                foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCLineDTOData.ItemBarCodes)
                {
                    if (shipLine.WHKey == null)
                    {
                        throw new ApplicationException(string.Format("料码{0}(料品{1}行{2})没有指定库位，无法确定存储地点，请分配库位", shipPickConfirmItemBCDTOData.ItemBarCode, shipLine.ItemInfo.ItemName, shipLine.DocLineNo));
                    }
                    foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in shipPickConfirmBCLineDTOData.BinLines)
                    {
                        if (shipLine.WHKey.ID == shipPickConfirmBCBinDTOData.WHKey)
                        {
                            throw new ApplicationException(string.Format("料码{0}(料品{1}行{2})没有指定库位，存储地点是库位管理的，请分配库位", shipPickConfirmItemBCDTOData.ItemBarCode, shipLine.ItemInfo.ItemName, shipLine.DocLineNo));
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 设置默认值
        /// </summary>
        /// <param name="dto"></param>
        private void SetDefaultValue(ShipPickConfirmBCDTOData dto)
        {
            foreach (ShipPickConfirmBCLineDTOData shipPickConfirmBCLineDTOData in dto.DocLines)
            {
                ShipLine shipLine = ShipLine.Finder.FindByID(shipPickConfirmBCLineDTOData.DocLineKey);
                shipPickConfirmBCLineDTOData.WHKey = -1L;
                if (shipLine.WHKey != null)
                {
                    shipPickConfirmBCLineDTOData.WHKey = shipLine.WHKey.ID;
                }
                foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCLineDTOData.ItemBarCodes)
                {
                    shipPickConfirmItemBCDTOData.ShipLineKey = shipPickConfirmBCLineDTOData.DocLineKey;
                    shipPickConfirmItemBCDTOData.ShipPickConfirmBCLineDTO = shipPickConfirmBCLineDTOData;
                    if (string.IsNullOrEmpty(shipPickConfirmItemBCDTOData.LotCode))
                    {
                        shipPickConfirmItemBCDTOData.LotCode = "";
                    }
                }
                foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in shipPickConfirmBCLineDTOData.BinLines)
                {
                    foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData2 in shipPickConfirmBCBinDTOData.ItemBarCodes)
                    {
                        shipPickConfirmItemBCDTOData2.ShipLineKey = shipPickConfirmBCLineDTOData.DocLineKey;
                        shipPickConfirmItemBCDTOData2.ShipPickConfirmBCBinDTO = shipPickConfirmBCBinDTOData;
                        if (string.IsNullOrEmpty(shipPickConfirmItemBCDTOData2.LotCode))
                        {
                            shipPickConfirmItemBCDTOData2.LotCode = "";
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 修正出货单
        /// </summary>
        /// <param name="dto"></param>
        private void ModifyShip(ShipPickConfirmBCDTOData dto)
        {
            int systemLineNO = SMTools.GetSystemLineNO();
            Dictionary<long, Dictionary<long, List<ShipPickConfirmBCBinDTOData>>> dictionary = new Dictionary<long, Dictionary<long, List<ShipPickConfirmBCBinDTOData>>>();
            Dictionary<long, Dictionary<string, List<ShipPickConfirmItemBCDTOData>>> dictionary2 = new Dictionary<long, Dictionary<string, List<ShipPickConfirmItemBCDTOData>>>();
            ShipDTOForIndustryChainData shipDTOForIndustryChainData = new ShipDTOForIndustryChainData();
            shipDTOForIndustryChainData.ID = dto.DocKey;
            shipDTOForIndustryChainData.ChangeFieldList.Add("ID");
            shipDTOForIndustryChainData.ShipLines = new List<ShipLineDTOForIndustryChainData>();
            List<long> list = new List<long>();
            foreach (ShipPickConfirmBCLineDTOData shipPickConfirmBCLineDTOData in dto.DocLines)
            {
                ShipLine shipLine = ShipLine.Finder.FindByID(shipPickConfirmBCLineDTOData.DocLineKey);
                if (shipLine == null)
                {
                    throw new Exception(string.Format("出货单上已经不存在此行(key:[0], qty:[1])已经不存在，请移除", shipPickConfirmBCLineDTOData.DocLineKey, shipPickConfirmBCLineDTOData.Qty));
                }
                list.Add(shipPickConfirmBCLineDTOData.DocLineKey);
                if (shipPickConfirmBCLineDTOData.Qty == 0m)
                {
                    ShipLineDTOForIndustryChainData shipLineDTOForIndustryChainData = new ShipLineDTOForIndustryChainData();
                    shipLineDTOForIndustryChainData.ID = shipPickConfirmBCLineDTOData.DocLineKey;
                    shipLineDTOForIndustryChainData.DeleteFlag = true;
                    shipDTOForIndustryChainData.ShipLines.Add(shipLineDTOForIndustryChainData);
                }
                else
                {
                    if (shipPickConfirmBCLineDTOData.DescFlexFields != null && shipPickConfirmBCLineDTOData.DescFlexFields.Count > 0)
                    {
                        using (Session session = Session.Open())
                        {
                            foreach (DescFlexFieldDTOData descFlexFieldDTOData in shipPickConfirmBCLineDTOData.DescFlexFields)
                            {
                                string flexFieldName = descFlexFieldDTOData.FlexFieldName;
                                string flexFieldValue = descFlexFieldDTOData.FlexFieldValue;
                                if (!string.IsNullOrEmpty(flexFieldName))
                                {
                                    PropertyInfo property = shipLine.DescFlexField.GetType().GetProperty(flexFieldName);
                                    if (property != null)
                                    {
                                        property.SetValue(shipLine.DescFlexField, flexFieldValue);
                                    }
                                }
                            }
                            session.Commit();
                        }
                    }
                    long num = -1L;
                    if (shipLine.WHKey != null)
                    {
                        num = shipLine.WHKey.ID;
                    }
                    bool flag = false;
                    bool flag2 = false;
                    Dictionary<string, List<ShipPickConfirmItemBCDTOData>> dictionary3 = new Dictionary<string, List<ShipPickConfirmItemBCDTOData>>();
                    if (string.IsNullOrEmpty(shipLine.LotInfo.LotCode))
                    {
                        dictionary3 = this.BuildLotItems(shipPickConfirmBCLineDTOData);
                        if (dictionary3 != null && dictionary3.Keys.Count > 0)
                        {
                            string a = "";
                            if (dictionary3.Keys.Count == 1)
                            {
                                foreach (string text in dictionary3.Keys)
                                {
                                    a = text;
                                }
                            }
                            if (a != shipLine.LotInfo.LotCode || dictionary3.Keys.Count > 1)
                            {
                                dictionary2.Add(shipPickConfirmBCLineDTOData.DocLineKey, dictionary3);
                                foreach (string text in dictionary3.Keys)
                                {
                                    foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in dictionary3[text])
                                    {
                                        if (shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO != null)
                                        {
                                            shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO.IsSplitted = true;
                                        }
                                    }
                                }
                                flag = true;
                            }
                        }
                    }
                    Dictionary<long, List<ShipPickConfirmBCBinDTOData>> dictionary4 = this.BuildWHBins(shipLine, shipPickConfirmBCLineDTOData.BinLines, shipPickConfirmBCLineDTOData.ItemBarCodes, false);
                    if (dictionary4.Keys.Count > 1)
                    {
                        dictionary.Add(shipPickConfirmBCLineDTOData.DocLineKey, dictionary4);
                        flag2 = true;
                    }
                    if (!flag2)
                    {
                        if (flag)
                        {
                            if (num <= 0L && (shipPickConfirmBCLineDTOData.BinLines.Count == 0 || shipPickConfirmBCLineDTOData.BinLines[0].WHKey <= 0L))
                            {
                                continue;
                            }
                        }
                        ShipLineDTOForIndustryChainData shipLineDTOForIndustryChainData = new ShipLineDTOForIndustryChainData();
                        shipLineDTOForIndustryChainData.ID = shipPickConfirmBCLineDTOData.DocLineKey;
                        shipLineDTOForIndustryChainData.ChangeFieldList = new List<string>();
                        shipLineDTOForIndustryChainData.ChangeFieldList.Add("ID");
                        shipDTOForIndustryChainData.ShipLines.Add(shipLineDTOForIndustryChainData);
                        shipLineDTOForIndustryChainData.ShipLineLocations = new List<ShipLineLocationDTOForIndustryChainData>();
                        int num2 = 0;
                        for (int i = shipLine.ShipLineLocations.Count; i > 0; i--)
                        {
                            ShipLineLocation shipLineLocation = shipLine.ShipLineLocations[i - 1];
                            ShipLineLocationDTOForIndustryChainData shipLineLocationDTOForIndustryChainData = new ShipLineLocationDTOForIndustryChainData();
                            shipLineLocationDTOForIndustryChainData.ID = shipLineLocation.ID;
                            shipLineLocationDTOForIndustryChainData.DeleteFlag = true;
                            shipLineDTOForIndustryChainData.ShipLineLocations.Add(shipLineLocationDTOForIndustryChainData);
                        }
                        if (shipPickConfirmBCLineDTOData.Qty != shipLine.ShipQtyInvAmount)
                        {
                            shipLineDTOForIndustryChainData.ShipQtyInvAmount = shipPickConfirmBCLineDTOData.Qty;
                            shipLineDTOForIndustryChainData.ShipQtyTUAmount = 0m;
                            shipLineDTOForIndustryChainData.ShipQtyTBUAmount = 0m;
                            shipLineDTOForIndustryChainData.ChangeFieldList.Add("ShipQtyInvAmount");
                            shipLineDTOForIndustryChainData.ChangeFieldList.Add("ShipQtyTUAmount");
                            shipLineDTOForIndustryChainData.ChangeFieldList.Add("ShipQtyTBUAmount");
                        }
                        foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in shipPickConfirmBCLineDTOData.BinLines)
                        {
                            ShipLineLocationDTOForIndustryChainData shipLineLocationDTOForIndustryChainData = new ShipLineLocationDTOForIndustryChainData();
                            shipLineLocationDTOForIndustryChainData.LocationQtyInvAmount = shipPickConfirmBCBinDTOData.Qty;
                            num2 = (shipLineLocationDTOForIndustryChainData.LocationNo = num2 + systemLineNO);
                            shipLineLocationDTOForIndustryChainData.Location = new CommonArchiveDataDTOData();
                            shipLineLocationDTOForIndustryChainData.Location.ID = shipPickConfirmBCBinDTOData.BinKey;
                            shipLineLocationDTOForIndustryChainData.ID = (long)(-(long)shipLineLocationDTOForIndustryChainData.LocationNo);
                            shipLineDTOForIndustryChainData.ShipLineLocations.Add(shipLineLocationDTOForIndustryChainData);
                        }
                        if (shipPickConfirmBCLineDTOData.BinLines.Count > 0)
                        {
                            shipLineDTOForIndustryChainData.WH = new CommonArchiveDataDTOData(shipPickConfirmBCLineDTOData.BinLines[0].WHKey, "", "");
                            shipLineDTOForIndustryChainData.ChangeFieldList.Add("WH");
                        }
                    }
                }
            }
            Ship ship = Ship.Finder.FindByID(dto.DocKey);
            foreach (ShipLine shipLine in ship.ShipLines)
            {
                if (!BCHelper.IsNoPickLine(shipLine))
                {
                    if (!list.Contains(shipLine.ID))
                    {
                        ShipLineDTOForIndustryChainData shipLineDTOForIndustryChainData = new ShipLineDTOForIndustryChainData();
                        shipLineDTOForIndustryChainData.ID = shipLine.ID;
                        shipLineDTOForIndustryChainData.DeleteFlag = true;
                        shipDTOForIndustryChainData.ShipLines.Add(shipLineDTOForIndustryChainData);
                    }
                }
            }
            List<ShipLineSplitDTOData> list2 = new List<ShipLineSplitDTOData>();
            list2.AddRange(this.BuildSplitDTOByLot(dictionary2));
            list2.AddRange(this.BuildSplitDTOByWH(dictionary));
            if (shipDTOForIndustryChainData.ShipLines.Count > 0)
            {
                new UpdateShipSVProxy
                {
                    ShipDTOs = new List<ShipDTOForIndustryChainData>()
                    {
                        shipDTOForIndustryChainData
                    }
                }.Do();
            }
            if (list2.Count > 0)
            {
                List<ShipLineSplitDTOData> splitlineresults = new ShipLineSplitSVProxy
                {
                    ShipSplitLines = list2
                }.Do();
                this.UpdateItemBarCodeSrcIndex(splitlineresults, dictionary, dictionary2);
            }
            PLCacheManager.DataCache.FlushCache();
            PLCacheManager.ObjectCache.FlushCache();
        }

        /// <summary>
        /// 更新行线索
        /// </summary>
        /// <param name="splitlineresults"></param>
        /// <param name="shipline_wh_bins"></param>
        /// <param name="shipline_lot_items"></param>
        // Token: 0x0600023B RID: 571 RVA: 0x0001ED28 File Offset: 0x0001CF28
        private void UpdateItemBarCodeSrcIndex(List<ShipLineSplitDTOData> splitlineresults, Dictionary<long, Dictionary<long, List<ShipPickConfirmBCBinDTOData>>> shipline_wh_bins, Dictionary<long, Dictionary<string, List<ShipPickConfirmItemBCDTOData>>> shipline_lot_items)
        {
            foreach (ShipLineSplitDTOData shipLineSplitDTOData in splitlineresults)
            {
                if (shipline_wh_bins.ContainsKey(shipLineSplitDTOData.ShipLineKey))
                {
                    foreach (ShipLineSplitDetailDTOData shipLineSplitDetailDTOData in shipLineSplitDTOData.SplitDetails)
                    {
                        foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in shipline_wh_bins[shipLineSplitDTOData.ShipLineKey][shipLineSplitDetailDTOData.WHKey])
                        {
                            foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCBinDTOData.ItemBarCodes)
                            {
                                shipPickConfirmItemBCDTOData.ShipLineKey = shipLineSplitDetailDTOData.ShipLineKey;
                            }
                        }
                    }
                }
                else if (shipline_lot_items.ContainsKey(shipLineSplitDTOData.ShipLineKey))
                {
                    foreach (ShipLineSplitDetailDTOData shipLineSplitDetailDTOData in shipLineSplitDTOData.SplitDetails)
                    {
                        foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipline_lot_items[shipLineSplitDTOData.ShipLineKey][shipLineSplitDetailDTOData.LotCode])
                        {
                            if (shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO != null)
                            {
                                if (shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO.WHKey == shipLineSplitDetailDTOData.WHKey)
                                {
                                    shipPickConfirmItemBCDTOData.ShipLineKey = shipLineSplitDetailDTOData.ShipLineKey;
                                }
                            }
                            else if (shipPickConfirmItemBCDTOData.ShipPickConfirmBCLineDTO != null)
                            {
                                if (shipPickConfirmItemBCDTOData.ShipPickConfirmBCLineDTO.WHKey == shipLineSplitDetailDTOData.WHKey)
                                {
                                    shipPickConfirmItemBCDTOData.ShipLineKey = shipLineSplitDetailDTOData.ShipLineKey;
                                }
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 构建Lot_Items
        /// </summary>
        /// <param name="linedto"></param>
        /// <returns></returns>
        // Token: 0x0600023C RID: 572 RVA: 0x0001F018 File Offset: 0x0001D218
        private Dictionary<string, List<ShipPickConfirmItemBCDTOData>> BuildLotItems(ShipPickConfirmBCLineDTOData linedto)
        {
            Dictionary<string, List<ShipPickConfirmItemBCDTOData>> dictionary = new Dictionary<string, List<ShipPickConfirmItemBCDTOData>>();
            foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in linedto.BinLines)
            {
                foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCBinDTOData.ItemBarCodes)
                {
                    if (!dictionary.ContainsKey(shipPickConfirmItemBCDTOData.LotCode))
                    {
                        dictionary.Add(shipPickConfirmItemBCDTOData.LotCode, new List<ShipPickConfirmItemBCDTOData>());
                    }
                    dictionary[shipPickConfirmItemBCDTOData.LotCode].Add(shipPickConfirmItemBCDTOData);
                }
            }
            foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData2 in linedto.ItemBarCodes)
            {
                if (!dictionary.ContainsKey(shipPickConfirmItemBCDTOData2.LotCode))
                {
                    dictionary.Add(shipPickConfirmItemBCDTOData2.LotCode, new List<ShipPickConfirmItemBCDTOData>());
                }
                dictionary[shipPickConfirmItemBCDTOData2.LotCode].Add(shipPickConfirmItemBCDTOData2);
            }
            return dictionary;
        }

        /// <summary>
        /// 构建Lot-Bins
        /// </summary>
        /// <param name="lot_items"></param>
        /// <param name="lot_bindtos"></param>
        /// <param name="lot_nobinitems"></param>
        // Token: 0x0600023D RID: 573 RVA: 0x0001F17C File Offset: 0x0001D37C
        private void BuildLotBins(Dictionary<string, List<ShipPickConfirmItemBCDTOData>> lot_items, ref Dictionary<string, List<ShipPickConfirmBCBinDTOData>> lot_bindtos, ref Dictionary<string, List<ShipPickConfirmItemBCDTOData>> lot_nobinitems)
        {
            Dictionary<string, List<long>> dictionary = new Dictionary<string, List<long>>();
            foreach (string key in lot_items.Keys)
            {
                dictionary.Add(key, new List<long>());
                lot_bindtos.Add(key, new List<ShipPickConfirmBCBinDTOData>());
                lot_nobinitems.Add(key, new List<ShipPickConfirmItemBCDTOData>());
                Dictionary<long, ShipPickConfirmBCBinDTOData> dictionary2 = new Dictionary<long, ShipPickConfirmBCBinDTOData>();
                foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in lot_items[key])
                {
                    if (shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO != null)
                    {
                        long binKey = shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO.BinKey;
                        if (!dictionary[key].Contains(binKey))
                        {
                            dictionary[key].Add(binKey);
                            ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData = new ShipPickConfirmBCBinDTOData();
                            lot_bindtos[key].Add(shipPickConfirmBCBinDTOData);
                            shipPickConfirmBCBinDTOData.WHKey = shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO.WHKey;
                            shipPickConfirmBCBinDTOData.BinKey = shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO.BinKey;
                            shipPickConfirmBCBinDTOData.BinLineKey = shipPickConfirmItemBCDTOData.ShipPickConfirmBCBinDTO.BinLineKey;
                            shipPickConfirmBCBinDTOData.ItemBarCodes = new List<ShipPickConfirmItemBCDTOData>();
                            shipPickConfirmBCBinDTOData.ItemBarCodes.Add(shipPickConfirmItemBCDTOData);
                            dictionary2.Add(binKey, shipPickConfirmBCBinDTOData);
                        }
                        else
                        {
                            dictionary2[binKey].ItemBarCodes.Add(shipPickConfirmItemBCDTOData);
                        }
                    }
                    else
                    {
                        lot_nobinitems[key].Add(shipPickConfirmItemBCDTOData);
                    }
                }
            }
        }

        /// <summary>
        /// 构建WH-Bins
        /// </summary>
        /// <param name="shipline"></param>
        /// <param name="bins"></param>
        /// <param name="nobinitems"></param>
        /// <returns></returns>
        // Token: 0x0600023E RID: 574 RVA: 0x0001F35C File Offset: 0x0001D55C
        private Dictionary<long, List<ShipPickConfirmBCBinDTOData>> BuildWHBins(ShipLine shipline, List<ShipPickConfirmBCBinDTOData> bins, List<ShipPickConfirmItemBCDTOData> nobinitems, bool splitlot)
        {
            Dictionary<long, List<ShipPickConfirmBCBinDTOData>> dictionary = new Dictionary<long, List<ShipPickConfirmBCBinDTOData>>();
            foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in bins)
            {
                if (!shipPickConfirmBCBinDTOData.IsSplitted)
                {
                    shipPickConfirmBCBinDTOData.ShipLineKey = shipline.ID;
                    if (shipline.WHKey == null || shipPickConfirmBCBinDTOData.WHKey != shipline.WHKey.ID || splitlot)
                    {
                        if (!dictionary.ContainsKey(shipPickConfirmBCBinDTOData.WHKey))
                        {
                            dictionary.Add(shipPickConfirmBCBinDTOData.WHKey, new List<ShipPickConfirmBCBinDTOData>());
                        }
                        dictionary[shipPickConfirmBCBinDTOData.WHKey].Add(shipPickConfirmBCBinDTOData);
                    }
                }
            }
            if (nobinitems.Count > 0)
            {
                dictionary.Add(shipline.WHKey.ID, new List<ShipPickConfirmBCBinDTOData>());
                ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData = new ShipPickConfirmBCBinDTOData();
                dictionary[shipline.WHKey.ID].Add(shipPickConfirmBCBinDTOData);
                shipPickConfirmBCBinDTOData.ItemBarCodes = new List<ShipPickConfirmItemBCDTOData>();
                foreach (ShipPickConfirmItemBCDTOData item in nobinitems)
                {
                    shipPickConfirmBCBinDTOData.ItemBarCodes.Add(item);
                }
            }
            return dictionary;
        }

        /// <summary>
        /// 审核出货单
        /// </summary>
        /// <param name="dto"></param>
        // Token: 0x0600023F RID: 575 RVA: 0x0001F4F0 File Offset: 0x0001D6F0
        private void ApproveShip(ShipPickConfirmBCDTOData dto)
        {
            this.DocApprove(dto);
            this.FlowApprove(dto);
        }

        // Token: 0x06000240 RID: 576 RVA: 0x0001F504 File Offset: 0x0001D704
        private void DocApprove(ShipPickConfirmBCDTOData dto)
        {
            Ship ship = Ship.Finder.FindByID(dto.DocKey);
            if (ship.DocumentType.ConfirmType != ConfirmTypeEnum.ApproveFlow)
            {
                AuditShipSVProxy auditShipSVProxy = new AuditShipSVProxy();
                auditShipSVProxy.ShipKeys = new List<DocKeyDTOData>();
                if (ship.Status == ShipStateEnum.Creates)
                {
                    auditShipSVProxy.IsSubmitAndApprove = true;
                }
                DocKeyDTOData docKeyDTOData = new DocKeyDTOData();
                docKeyDTOData.DocID = dto.DocKey;
                auditShipSVProxy.ShipKeys.Add(docKeyDTOData);
                auditShipSVProxy.Do();
            }
        }

        // Token: 0x06000241 RID: 577 RVA: 0x0001F59C File Offset: 0x0001D79C
        private void FlowApprove(ShipPickConfirmBCDTOData dto)
        {
            Ship ship = Ship.Finder.FindByID(dto.DocKey);
            if (ship.DocumentType.ConfirmType == ConfirmTypeEnum.ApproveFlow)
            {
                BatchApproveBPProxy batchApproveBPProxy = new BatchApproveBPProxy();
                batchApproveBPProxy.ApproveActions = new List<BatchApproveActionDTOData>();
                BatchApproveActionDTOData batchApproveActionDTOData = new BatchApproveActionDTOData();
                BusinessEntity.EntityKey entityID = new BusinessEntity.EntityKey(dto.DocKey, ship.Key.EntityType);
                batchApproveActionDTOData.EntityID = entityID;
                batchApproveActionDTOData.Action = 1;
                batchApproveActionDTOData.ApproveType = 3;
                batchApproveActionDTOData.OperatorId = long.Parse(PlatformContext.Current.UserID);
                batchApproveBPProxy.ApproveActions.Add(batchApproveActionDTOData);
                List<BatchApproveResultData> list = batchApproveBPProxy.Do();
                if (list != null && list.Count > 0)
                {
                    if (!list[0].IsSuccessful)
                    {
                        throw new ApplicationException(list[0].ApproveDetail);
                    }
                }
            }
        }

        /// <summary>
        /// 构建拆分DTO
        /// </summary>
        /// <param name="shipline_wh_bins"></param>
        /// <returns></returns>
        // Token: 0x06000242 RID: 578 RVA: 0x0001F694 File Offset: 0x0001D894
        private List<ShipLineSplitDTOData> BuildSplitDTOByWH(Dictionary<long, Dictionary<long, List<ShipPickConfirmBCBinDTOData>>> shipline_wh_bins)
        {
            List<ShipLineSplitDTOData> list = new List<ShipLineSplitDTOData>();
            foreach (long num in shipline_wh_bins.Keys)
            {
                ShipLineSplitDTOData shipLineSplitDTOData = new ShipLineSplitDTOData();
                list.Add(shipLineSplitDTOData);
                shipLineSplitDTOData.ShipLineKey = num;
                shipLineSplitDTOData.SplitByBin = true;
                shipLineSplitDTOData.SplitDetails = new List<ShipLineSplitDetailDTOData>();
                this.BuildSplitDetailDTO(shipLineSplitDTOData, num, shipline_wh_bins[num]);
            }
            return list;
        }

        // Token: 0x06000243 RID: 579 RVA: 0x0001F730 File Offset: 0x0001D930
        private List<ShipLineSplitDTOData> BuildSplitDTOByLot(Dictionary<long, Dictionary<string, List<ShipPickConfirmItemBCDTOData>>> shipline_lot_items)
        {
            List<ShipLineSplitDTOData> list = new List<ShipLineSplitDTOData>();
            foreach (long num in shipline_lot_items.Keys)
            {
                Dictionary<string, List<ShipPickConfirmBCBinDTOData>> dictionary = new Dictionary<string, List<ShipPickConfirmBCBinDTOData>>();
                Dictionary<string, List<ShipPickConfirmItemBCDTOData>> dictionary2 = new Dictionary<string, List<ShipPickConfirmItemBCDTOData>>();
                ShipLine shipline = ShipLine.Finder.FindByID(num);
                this.BuildLotBins(shipline_lot_items[num], ref dictionary, ref dictionary2);
                ShipLineSplitDTOData shipLineSplitDTOData = new ShipLineSplitDTOData();
                list.Add(shipLineSplitDTOData);
                shipLineSplitDTOData.ShipLineKey = num;
                shipLineSplitDTOData.SplitByBin = true;
                shipLineSplitDTOData.SplitDetails = new List<ShipLineSplitDetailDTOData>();
                foreach (string key in dictionary.Keys)
                {
                    List<ShipPickConfirmBCBinDTOData> bins = dictionary[key];
                    List<ShipPickConfirmItemBCDTOData> nobinitems = dictionary2[key];
                    Dictionary<long, List<ShipPickConfirmBCBinDTOData>> wh_bins = this.BuildWHBins(shipline, bins, nobinitems, true);
                    this.BuildSplitDetailDTO(shipLineSplitDTOData, num, wh_bins);
                }
            }
            return list;
        }

        /// <summary>
        /// 构建拆分明细DTO
        /// </summary>
        /// <param name="splitline"></param>
        /// <param name="shipline"></param>
        /// <param name="wh_bins"></param>
        // Token: 0x06000244 RID: 580 RVA: 0x0001F870 File Offset: 0x0001DA70
        private void BuildSplitDetailDTO(ShipLineSplitDTOData splitline, long shipline, Dictionary<long, List<ShipPickConfirmBCBinDTOData>> wh_bins)
        {
            if (splitline.SplitDetails == null)
            {
                splitline.SplitDetails = new List<ShipLineSplitDetailDTOData>();
            }
            foreach (long num in wh_bins.Keys)
            {
                ShipLineSplitDetailDTOData shipLineSplitDetailDTOData = new ShipLineSplitDetailDTOData();
                splitline.SplitDetails.Add(shipLineSplitDetailDTOData);
                shipLineSplitDetailDTOData.WHKey = num;
                shipLineSplitDetailDTOData.Bins = new List<ShipLineSplitBinDetailDTOData>();
                shipLineSplitDetailDTOData.ShipQtyIU = 0m;
                foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in wh_bins[num])
                {
                    ShipLineSplitBinDetailDTOData shipLineSplitBinDetailDTOData = new ShipLineSplitBinDetailDTOData();
                    shipLineSplitBinDetailDTOData.BinKey = shipPickConfirmBCBinDTOData.BinKey;
                    shipLineSplitBinDetailDTOData.Qty = 0m;
                    if (shipPickConfirmBCBinDTOData.BinKey > 0L)
                    {
                        shipLineSplitDetailDTOData.Bins.Add(shipLineSplitBinDetailDTOData);
                    }
                    foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCBinDTOData.ItemBarCodes)
                    {
                        shipLineSplitBinDetailDTOData.Qty += shipPickConfirmItemBCDTOData.Qty;
                        shipLineSplitDetailDTOData.ShipQtyIU += shipPickConfirmItemBCDTOData.Qty;
                        shipLineSplitDetailDTOData.LotCode = shipPickConfirmItemBCDTOData.LotCode;
                    }
                }
            }
        }

        // Token: 0x06000245 RID: 581 RVA: 0x0001FA5C File Offset: 0x0001DC5C
        private void UpdateBarCodeState(ShipPickConfirmBCDTOData dto)
        {
            if (!string.IsNullOrEmpty(dto.BarCode))
            {
                List<string> list = new List<string>();
                list.Add(dto.BarCode);
                if (list.Count > 0)
                {
                    new UpdateBarCodeStateProxy
                    {
                        BarCodes = list
                    }.Do();
                }
            }
        }

        // Token: 0x06000246 RID: 582 RVA: 0x0001FAB8 File Offset: 0x0001DCB8
        private void CreateItemBarCodeUseDetail(ShipPickConfirmBCDTOData dto)
        {
            List<CreateBarCodeUsedInDTOData> list = new List<CreateBarCodeUsedInDTOData>();
            ShipLine.EntityList entityList = ShipLine.Finder.FindAll("Ship=@ship", new OqlParam[]
            {
                new OqlParam("ship", dto.DocKey)
            });
            foreach (ShipPickConfirmBCLineDTOData shipPickConfirmBCLineDTOData in dto.DocLines)
            {
                foreach (ShipPickConfirmBCBinDTOData shipPickConfirmBCBinDTOData in shipPickConfirmBCLineDTOData.BinLines)
                {
                    foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCBinDTOData.ItemBarCodes)
                    {
                        if (shipPickConfirmBCLineDTOData.Qty == 0m)
                        {
                            throw new Exception(string.Format("条码{0}使用异常：对应的单行实际出货数量为0，并没有出货，请移除", shipPickConfirmItemBCDTOData.Qty));
                        }
                        ShipLine shipline = ShipLine.Finder.FindByID(shipPickConfirmItemBCDTOData.ShipLineKey);
                        ShipLineLocation.EntityList entityList2 = ShipLineLocation.Finder.FindAll("ShipLine=@shipline", new OqlParam[]
                        {
                            new OqlParam("shipline", shipPickConfirmItemBCDTOData.ShipLineKey)
                        });
                        foreach (ShipLineLocation shipLineLocation in entityList2)
                        {
                            if (shipLineLocation.LocationKey.ID == shipPickConfirmBCBinDTOData.BinKey)
                            {
                                CreateBarCodeUsedInDTOData commonBCUseDTO = this.GetCommonBCUseDTO(shipline);
                                list.Add(commonBCUseDTO);
                                commonBCUseDTO.SubLine = new BizEntityKeyData();
                                commonBCUseDTO.SubLine.EntityID = shipLineLocation.ID;
                                commonBCUseDTO.SubLine.EntityType = shipLineLocation.Key.EntityType;
                                commonBCUseDTO.BarCode = shipPickConfirmItemBCDTOData.ItemBarCode;
                                commonBCUseDTO.CurQty = shipPickConfirmItemBCDTOData.Qty;
                                if (!string.IsNullOrEmpty(shipPickConfirmItemBCDTOData.BoxBarCode))
                                {
                                    CreateBarCodeUsedInDTOData commonBCUseDTO2 = this.GetCommonBCUseDTO(shipline);
                                    list.Add(commonBCUseDTO2);
                                    commonBCUseDTO2.SubLine = new BizEntityKeyData();
                                    commonBCUseDTO2.SubLine.EntityID = shipLineLocation.ID;
                                    commonBCUseDTO2.SubLine.EntityType = shipLineLocation.Key.EntityType;
                                    commonBCUseDTO2.BarCode = shipPickConfirmItemBCDTOData.BoxBarCode;
                                    commonBCUseDTO2.CurQty = 0m;
                                }
                            }
                        }
                    }
                }
                foreach (ShipPickConfirmItemBCDTOData shipPickConfirmItemBCDTOData in shipPickConfirmBCLineDTOData.ItemBarCodes)
                {
                    if (shipPickConfirmBCLineDTOData.Qty == 0m)
                    {
                        throw new Exception(string.Format("条码{0}使用异常：对应的单行实际出货数量为0，并没有出货，请移除", shipPickConfirmItemBCDTOData.Qty));
                    }
                    ShipLine shipline = ShipLine.Finder.FindByID(shipPickConfirmItemBCDTOData.ShipLineKey);
                    CreateBarCodeUsedInDTOData commonBCUseDTO = this.GetCommonBCUseDTO(shipline);
                    list.Add(commonBCUseDTO);
                    commonBCUseDTO.BarCode = shipPickConfirmItemBCDTOData.ItemBarCode;
                    commonBCUseDTO.CurQty = shipPickConfirmItemBCDTOData.Qty;
                    if (!string.IsNullOrEmpty(shipPickConfirmItemBCDTOData.BoxBarCode))
                    {
                        CreateBarCodeUsedInDTOData commonBCUseDTO2 = this.GetCommonBCUseDTO(shipline);
                        list.Add(commonBCUseDTO2);
                        commonBCUseDTO2.BarCode = shipPickConfirmItemBCDTOData.BoxBarCode;
                        commonBCUseDTO2.CurQty = 0m;
                    }
                }
            }
            if (list.Count > 0)
            {
                new CreateBarCodeUsedSVProxy
                {
                    CreateBCUsedInParams = list,
                    App = 14
                }.Do();
            }
        }

        // Token: 0x06000247 RID: 583 RVA: 0x0001FF48 File Offset: 0x0001E148
        private CreateBarCodeUsedInDTOData GetCommonBCUseDTO(ShipLine shipline)
        {
            return new CreateBarCodeUsedInDTOData
            {
                Doc = new BizEntityKeyData()
                {
                    EntityID = shipline.ShipKey.ID,
                    EntityType = shipline.ShipKey.EntityType
                },
                DocLine = new BizEntityKeyData()
                {
                    EntityID = shipline.ID,
                    EntityType = shipline.Key.EntityType
                },
                Item = shipline.ItemInfo.ItemIDKey.ID,
                ItemCode = shipline.ItemInfo.ItemCode,
                ItemName = shipline.ItemInfo.ItemName,
                Org = Context.LoginOrg.ID,
                DocNo = shipline.Ship.DocNo,
                DocLineNo = shipline.DocLineNo.ToString(),
                DocOrg = shipline.OrgKey.ID,
                Direct = DirectEnum.Shipment.Value,
                IsUpdateWh = true
            };
        }
    }

	#endregion
	
	
}